library benchpress.src.sampler_dart;

import 'package:angular2/src/facade/lang.dart'
    show isPresent, isBlank, DateTime, DateWrapper;
import 'package:angular2/src/facade/async.dart' show Future, PromiseWrapper;
import 'package:angular2/src/facade/collection.dart'
    show StringMapWrapper, Map, List, ListWrapper;
import 'package:angular2/di.dart' show bind, OpaqueToken;
import './metric.dart' show Metric;
import './validator.dart' show Validator;
import './reporter.dart' show Reporter;
import './web_driver_extension.dart' show WebDriverExtension;
import './web_driver_adapter.dart' show WebDriverAdapter;
import './sample_options.dart' show Options;
import './measure_values.dart' show MeasureValues;

class Sampler {
  static get BINDINGS {
    return _BINDINGS;
  }
  static get TIME {
    return _TIME;
  }
  WebDriverAdapter _driver;
  WebDriverExtension _driverExtension;
  Metric _metric;
  Reporter _reporter;
  Validator _validator;
  bool _forceGc;
  Function _prepare;
  Function _execute;
  Function _time;
  Sampler({driver, driverExtension, metric, reporter, validator, forceGc,
      prepare, execute, time}) {
    this._driver = driver;
    this._driverExtension = driverExtension;
    this._metric = metric;
    this._reporter = reporter;
    this._validator = validator;
    this._forceGc = forceGc;
    this._prepare = prepare;
    this._execute = execute;
    this._time = time;
  }
  Future<SampleState> sample() {
    var loop;
    loop = (lastState) {
      return this._iterate(lastState).then((newState) {
        if (isPresent(newState.validSample)) {
          return newState;
        } else {
          return loop(newState);
        }
      });
    };
    return this._gcIfNeeded().then((_) {
      return loop(new SampleState([], null));
    });
  }
  _gcIfNeeded() {
    if (this._forceGc) {
      return this._driverExtension.gc();
    } else {
      return PromiseWrapper.resolve(null);
    }
  }
  _iterate(lastState) {
    var resultPromise;
    if (isPresent(this._prepare)) {
      resultPromise = this._driver.waitFor(this._prepare).then((_) {
        return this._gcIfNeeded();
      });
    } else {
      resultPromise = PromiseWrapper.resolve(null);
    }
    if (isPresent(this._prepare) ||
        identical(lastState.completeSample.length, 0)) {
      resultPromise = resultPromise.then((_) {
        return this._metric.beginMeasure();
      });
    }
    return resultPromise.then((_) {
      return this._driver.waitFor(this._execute);
    }).then((_) {
      return this._gcIfNeeded();
    }).then((_) {
      return this._metric.endMeasure(isBlank(this._prepare));
    }).then((measureValues) {
      return this._report(lastState, measureValues);
    });
  }
  Future<SampleState> _report(SampleState state, Map metricValues) {
    var measureValues = new MeasureValues(
        state.completeSample.length, this._time(), metricValues);
    var completeSample =
        ListWrapper.concat(state.completeSample, [measureValues]);
    var validSample = this._validator.validate(completeSample);
    var resultPromise = this._reporter.reportMeasureValues(measureValues);
    if (isPresent(validSample)) {
      resultPromise = resultPromise.then((_) {
        return this._reporter.reportSample(completeSample, validSample);
      });
    }
    return resultPromise.then((_) {
      return new SampleState(completeSample, validSample);
    });
  }
}
class SampleState {
  List completeSample;
  List validSample;
  SampleState(List completeSample, List validSample) {
    this.completeSample = completeSample;
    this.validSample = validSample;
  }
}
var _TIME = new OpaqueToken('Sampler.time');
var _BINDINGS = [
  bind(Sampler).toFactory((driver, driverExtension, metric, reporter, validator,
      forceGc, prepare, execute, time) {
    return new Sampler(
        driver: driver,
        driverExtension: driverExtension,
        reporter: reporter,
        validator: validator,
        metric: metric,
        forceGc: forceGc,
        prepare: !identical(prepare, false) ? prepare : null,
        execute: execute,
        time: time);
  }, [
    WebDriverAdapter,
    WebDriverExtension,
    Metric,
    Reporter,
    Validator,
    Options.FORCE_GC,
    Options.PREPARE,
    Options.EXECUTE,
    _TIME
  ]),
  bind(Options.FORCE_GC).toValue(false),
  bind(Options.PREPARE).toValue(false),
  bind(_TIME).toValue(() {
    return DateWrapper.now();
  })
];
